#
# Base32 encoder/decoder
# Based on an idea from Douglas Crockford
# http://www.crockford.com/wrmg/base32.html
#
# Author: Charlie Reiman
# Date: March 2003
#
# Issues:
#
# Crockford does not specify if the order is big endian or little
# endian. I assume big-endian.
#
# The check character is not optional
#

out_map_upper = "0123456789ABCDEFGHJKMNPQRSTVWXYZ" + "*#$=U"
out_map_lower = out_map_upper.lower()

in_map = {}

for i in range(len(out_map_upper)):
  in_map[out_map_upper[i]] = i
  in_map[out_map_lower[i]] = i
in_map['I'] = in_map['i'] = in_map['L'] = in_map['l'] = 1
in_map['O'] = in_map['o'] = 0

def encode_upper(in_val):
  return _encode(in_val, out_map_upper)

def encode_lower(in_val):
  return _encode(in_val, out_map_lower)

def _encode(in_val, out_map):
  assert len(out_map) == 37
  in_val = int(in_val)
  if in_val < 0:
    raise 'Base32: Negative numbers not supported'
  chk = in_val % 37
  result = []
  while 1:
    div_part = int(in_val/32)
    mod_part = in_val - div_part * 32
    result.append(out_map[mod_part])
    in_val = div_part
    if in_val == 0:
      break
  result.reverse()                      # Remove this for litte endian encoding
  result.append(out_map[chk])
  return ''.join(result)

def decode(in_str):
  in_str = in_str.replace('-','')
  chk = in_map[in_str[-1]]
  in_str = in_str[:-1]
  result = 0
  for ch in in_str:
    result *= 32
    result += in_map[ch]

  if result % 37 != chk:
    raise 'Base32: check failed.'
  return result

if __name__ == "__main__":
  for x in (0, 17, 31, 32, 33, 34, 174, 481120714, 2050000000):
    y = encode(x)
    print x, y
    if decode(y) != x:
      raise 'Fail 1'
    if decode(y.lower()) != x:
      raise 'Fail 2'
    if decode('-'+ y + '-') != x:
      raise 'Fail 3'
    if decode(y.replace('1','l')) != x:
      raise 'Fail 4'
    

