from OFS.SimpleItem import SimpleItem
from Globals import PersistentMapping
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

import Base32

import time

manage_addTeenyURLForm=PageTemplateFile('zpt/manage_addTeenyURLForm.html',
                                        globals())

def manage_addTeenyURL(self, ob_id="teeny", track="no",
                       inverse="no", REQUEST={}):
  """
  Add a TeenyURL instance.
  """
  ob = TeenyURL(ob_id, track!='no', inverse!='no')

  self._setObject(ob_id, ob, None, None, 0)
  self.__allow_groups__=self.acl_users
  ob = getattr(self, ob_id)

  if REQUEST:
    return self.manage_main(self, REQUEST)
  return ''

class Entry:
  "An entry in the db. Necessary to track usage"
  def __init__(self, val, track_use):
    self.val = val
    if track_use:
      self.start_tracking(time.time())

  def getval(self, track_use):
    if track_use:
      self.count += 1
      self.time = time.time()
    return self.val

  def clear_tracking(self):
    delattr(self, 'count')
    delattr(self, 'time')

  def start_tracking(self, time):
    self.count = 0
    self.time = time

class TeenyURL(SimpleItem):
  "Construct small URLs"

  meta_type="TeenyURL"
  manage_main = PageTemplateFile('zpt/manage_main.html', globals())
  manage_create_response = PageTemplateFile('zpt/manage_create_response.html',
                                            globals())
  manage_create = PageTemplateFile('zpt/manage_create.html',
                                   globals())
  manage_stats = PageTemplateFile('zpt/manage_stats.html', globals())
  manage_options = (
    {'label':'Edit', 'action':'manage_main'},
    {'label':'Stats', 'action':'manage_stats'}) + \
    SimpleItem.manage_options

  def __init__(self, id, track, inverse):
    "Make a new TeenyURL object"
    self.id = id
    self.url_num = 1
    self.map = PersistentMapping()
    self.track = track
    if inverse:
      self.inverse = PersistentMapping()
    else:
      self.inverse = 0

  def __before_publishing_traverse__(self, another_self, base_request):
    "Handle redirects in the smallest URL we can"

    if (len(base_request.path)==1):
      try:
        # Test decode and access
        x = Base32.decode(base_request.path[0])
        test = self.map[x]
      except:
        return
      # Looks good to me. Redirect to go method
      base_request.form['x'] = base_request.path[0]
      base_request['TraversalRequestNameStack']= ['go']

  def go(self, x='', REQUEST=None):
    "Go there. Track use if requested."
    target = self.map[Base32.decode(x)].getval(self.track)
    REQUEST.RESPONSE.redirect(target)

  def create(self, url='', REQUEST=None):
    "Create a teeny url, return an ugly, stock results webpage"
    if url != '':
      if REQUEST != None:
        coded_url = self.create_int(url = url)
        REQUEST.other['src_url'] = url
        REQUEST.other['coded_url_id'] = coded_url
        REQUEST.other['coded_url'] = self.absolute_url() + '/' + coded_url
        return self.manage_create_response(REQUEST)
      return None
    elif REQUEST != None:
      return self.manage_create(REQUEST)

  def create_url(self, url='', REQUEST=None):
    """Create a teeny url, return a url string to use it. For use
    in page templates or dtml."""
    coded_url = self.create_int(url = url)
    return self.absolute_url() + '/' + coded_url

  # Internal create method. Returns a string result. Not TTW callable
  # so no docstring provided.
  def create_int(self, url=''):
    # Check inverse map first.
    if self.inverse !=0 and self.inverse.has_key(url):
      result_num = self.inverse[url]
      # Update MRU date if tracked.
      if self.track:
        self.map[result_num].getval(self.track)
    else:
      result_num = self.url_num
      self.url_num += 1
      self.map[result_num] = Entry(url, self.track)
      # Update inverse if needed
      if self.inverse != 0:
        self.inverse[url] = result_num
    return Base32.encode_lower(result_num)

  def manage_edit(self, en_inverse="off", en_tracking="off", purge="0",
                  REQUEST=None):
    "Edit"
    resp = ''
    # Handle purges first, in case someone wants to delete the
    # tracking data as well
    if purge != "0" and self.track:
      cutoff = time.time()-float(purge)*86400
      for entry_key in self.map.keys():
        if self.map[entry_key].time < cutoff:
          # Note that updating the inverse table is trickier than you
          # might think
          if (self.inverse and self.inverse.has_key(self.map[entry_key]) and
              self.inverse[self.map[entry_key]] == entry_key):
            del self.inverse[self.map[entry_key]]
          del self.map[entry_key]
      resp += 'Purge successful.<br>'

    if en_inverse == "on" and self.inverse == 0:
      self.inverse = PersistentMapping()
      resp += 'Enabled inverse table<br>'
      for key in self.map.keys():
        self.inverse[self.map[key].getval(0)] = key
    elif en_inverse == "off" and self.inverse != 0:
      self.inverse = 0
      resp += 'Disabled inverse table<br>'

    if en_tracking=="on" and self.track == 0:
      self.track = 1
      now = time.time()
      for entry in self.map.values():
        entry.start_tracking(now)
      resp += 'Enabled tracking<br>'
    elif en_tracking=="off" and self.track:
      self.track = 0
      for entry in self.map.values():
        entry.clear_tracking()
      resp += 'Disabled tracking<br>'
    if REQUEST:
      resp = resp[:-4]
      return self.manage_main(self, REQUEST, manage_tabs_message=resp)

  def get_stats(self):
    "Return fun facts"
    stats = {}
    stats['oldest_id']='N/A'
    stats['oldest_url']='N/A'
    stats['newest_id']='N/A'
    stats['newest_url']='N/A'
    stats['active'] = str(len(self.map))
    stats['issued'] = str(self.url_num-1)
    stats['popular_id'] = 'N/A'
    stats['popular_url'] = 'N/A'
    stats['unpopular_id'] = 'N/A'
    stats['unpopular_url'] = 'N/A'
    map_keys = self.map.keys()
    map_keys.sort()
    # entrys are cronologically created, so oldest and newest
    # are at the endpoints, tracking or no.
    if len(map_keys) > 0:
      oldest = map_keys[0]
      newest = map_keys[-1]
      stats['oldest_id'] = Base32.encode_lower(oldest)
      stats['oldest_url'] = self.map[oldest].getval(0)
      stats['newest_id'] = Base32.encode_lower(newest)
      stats['newest_url'] = self.map[newest].getval(0)
      if self.track:
        popular = oldest
        unpopular = oldest
        for key in map_keys:
          if self.map[key].count > self.map[popular].count:
            popular = key
          if self.map[key].count < self.map[unpopular].count:
            unpopular = key
        stats['popular_id'] = Base32.encode_lower(popular)
        stats['popular_url'] = self.map[popular].getval(0)
        stats['unpopular_id'] = Base32.encode_lower(unpopular)
        stats['unpopular_url'] = self.map[unpopular].getval(0)

    return stats
